*! version 3.0.2 July 23, 2006 @ 23:38:21
*! runs error checking on a list of variables
program define ckvar , rclass
version 8
	/* 3.0.2 - added the slow option from dochar */
	/* 3.0.1 - changed name from checkvar to ckvar b/c there is already another checkvar */
	/* 3.0.0 - changed to run on validation rules rather than error rules */
	/*         direct error checking can be done ONLY by scoring (if wanted) or by using */
	/*         `error' as a temp variable when writing characteristic code */
	/* 2.2.1 - various bug fixes  */
	/* 2.2.0 - added ability to score a variable (as in grading a test) */
	/*       - scoring: missing values are assumed to have a score of 0 */
	/*       - for weights - scoring multiplies the marker by the weight, errors are still just 0,1,2 etc. */
	/* version 2.1.0 - split out the dochar program as separate */
	/* version 2.0.0 - uses characteristics to keep the error checking attached to variables */
	/* have checked the part which checks for missing values */
	/* have checked using another variable to check for missing values */
	/* have checked use of different missing value tags */
	/* have partially checked the use of labels */
	/*  - fix needed: need to figure smart way to drop existing value labels */
	/* have checked use of code, it's kinda clumsy, since it needs \$_self constructions */
	/*  - also need \$_error for holding the errors, or \$_score for holding scores */
	/* can use programs as long as they
	   `error' (or `score') is generated as all 0's to start
	then use a varlist and call `error'  */
	/* need to make output more readable */

	local myname "ckvar"
	set more 1
	syntax [varlist] [, KEY(varlist) MARKDup(namelist) TOTal(namelist) NOVars DROPLABELS STUB(str) SCore VALid keepgoing nopreserve progress slow]

	if "`valid'"!="" {
		if "`score'"!="" {
			display as error "`myname': Please specify either Score or Valid, but not both!"
			exit 198
			}
		}
	else {
		if "`score'"=="" {
			local valid "valid"
			}
		}

	if "`valid'"=="" {
		local what "score"
		}
	else {
		local what "error"
		}
	
	if "`total'"!="" {
		capture confirm new var `total'
		local rc = _rc
		if `rc' {
			if `rc'==110 {
				disp as error "`myname': The overall `what' total variable specified by total must be new!"
				exit 110
				}
			error `rc'
			}
		tempvar allmark
		gen byte `allmark' = 0
		if "`what'"=="error" {
			local s "s"							 /* s for stupid */
			}
		label var `allmark' "Total `what'`s' across the observations"
		}

	if "`key'"!="" {
		if "`markdup'"=="" {
			_ckdupl `key'
			}
		else {
			confirm new var `markdup'
			_ckdupl `key', gen(`markdup')
			}
		return scalar dups = r(dups)
		}

	if "`keepgoing'"!="" {
		local preserve "nopreserve"
		local keepgoing "_continue"
		}
	
	if "`novars'"=="" {
		if "`preserve'"=="" {
			preserve
			}
		capture n {
			if "`stub'"=="" {
				local varstub "`what'"
				local charstub "`what'"
				if "`valid'"!="" {
					local charstub "valid"
					}
				}
			else {
				local varstub "`stub'"
				local charstub "`stub'"
				}
		
			local cnt 1
			/* this makes the tempvars -error- and -valid- if error checking and -score- if scoring */
			tempvar `what' `valid'
			if "`valid'"!="" {
				local tempnames "valid:`valid' error:`error'"
				}
			else{
				local tempnames "score:`score'"
				}

			local maxScore 0
			local vcnt 1
			foreach self of local varlist {
				if "`progress'"!="" {
					display as text "Checking `self'..."
					}
				/* `self' will contain the name of the variable checked */
				local everythingOK 1
				local misval
				local checked 0
				local failreason

				/* dig through other like variables to find what vars are needed and what other vars are used */
				/*  this should help because it'll pick up required variables anywhere on the like chain */
				dolikedig `self', evalchar(`charstub'_check) datachar(`charstub'_other_vars_needed) accum
				local otherVarsNeeded "`r(contents)' `r(visits)'"
				local otherVarsNeeded: list uniq otherVarsNeeded
				local otherVarsNeeded: list otherVarsNeeded - self
			
				if "`otherVarsNeeded'"!="" {
					capture unab varsNeeded: `otherVarsNeeded'
					local rc = _rc
					if `rc' {
						if `rc'== 111 {
							display as error "The variables needed to check " as input "`self'" as error " for `what's are: " as input "`otherVarsNeeded'"
							local numvar : word count `otherVarsNeeded'
							if `numvar'>1 {
								local plu "these variables are"
								}
							else {
								local plu "this variable is"
								}
							display as error "Please make sure that `plu' available!"
							}
						if "`keepgoing'"=="" {
							exit `rc'
							}
						display as error "Did not check variable " as result `self'
						local failreason "needed vars missing"
						local everythingOK 0
						} /* end check for missing variable error */
					} /* end check for other vars */

				/* figure out the name of the score-holding variable */
				/* if scoring an exam, this would be used for question by question analysis - want for all variables checked */
				if `everythingOK' {
					/* find name to hold the scoring/errors for the variable in question */
					local scoreVar : char `self'[`varstub'_varname]
					if "`scoreVar'"=="" {
						local scoreVar = "`varstub'_" + substr("`self'",1,31-length("`stub'")-1)
						}
					capture confirm new var `scoreVar'
					local rc = _rc
					if `rc' {
						display as error "The variable for holding `what's from `self': " as result "`scoreVar'"
						display as error "  already exists!"
						if "`keepgoing'"=="" {
							exit `rc'
							}
						local everythingOK 0
						local failreason "`scoreVar' exists"
						}	/* end check of new scoring variable */
					}	/* end check if all ok */

				if `everythingOK' {
					/* run the validation/error check/scoring */
					capture dochar `self'[`charstub'_check], tempnames(`tempnames')
					if _rc {
						if "`keepgoing'"=="" {
							exit `rc'
							}
						else {
							local everythingOK 0
							local failreason "dochar failed"
							}
						}
					}

				if `everythingOK' {
					local checked = r(havechar) == "yes"
					
					/* negate valid values to get errors */
					if "`valid'"!="" & `checked' {
						capture confirm var `error'
						if _rc {
							gen byte `error' = !`valid'
							drop `valid'
							}
						}
				
					/* use the checking from the deepest variable visited */
					local ckVar : word 1 of `r(likeVarlist)'
					if "`ckVar'"=="" {
						local ckVar "`self'"
						}


					/* need to think about what to do here for scoring */
					local req : char `ckVar'[`charstub'_required]

					if "`req'"!="" & (("`req'" == "1") | strpos("true",lower("`req'")) | strpos("yes",lower("`req'"))) {
						local misval: char `ckVar'[`charstub'_missing_value]
					
						if "`misval'"=="" {
							local misval -1
							}

						/* because missing values could have the same errors as others: */
						quietly count if missing(`self')
						local misscount = r(N)
						if `misscount' { 
							capture confirm new variable ``what''
							if _rc {
								quietly replace ``what'' = `misval' if missing(`self')
								}
							else {
								/* need a slicker way of doing this */
								gen byte ``what'' = cond(missing(`self'),`misval',0)
								}
							local checked 1
							}	/* end of check of whether there were any missing values  */
						}		/* end of check for missing values important */
					else {
						capture confirm new variable ``what''
						if _rc {
							quietly replace ``what'' = 0 if missing(`self')
							}
						local misscount "N/A"
						} /* end check for separate missing */

					/* check to see if the scoreVar variable is even needed */
					/* --- will be kept if scoring, might be dropped when looking for errors */
			
					if `checked' {
						/* for error checking, drop temp var when all OK */
						if "`what'"=="error" {
							quietly count if ``what'' & (``what'' < .)
							local errcount = r(N)
							if `errcount'==0 {
								drop ``what''
								}
							}
						}	/* end of messages for checked variables */
					else {
						local misscount "N/A"
						local errcount "N/A"
						}

					/* now working with both scores and errors, but only if the the variable will be kept */
					capture confirm var ``what''
					if !_rc {
										
						local theWt : char `ckVar'[`stub'_wt]
						if "`theWt'"!="" {
							if `theWt'!=1 {
								if "`what'"=="score" {
									quietly replace ``what'' = ``what'' * `theWt'
									}
								else {
									local wtmod "weight of"
									}
								}
							}
						else {
							local theWt 1
							}

						local theLab : char `ckVar'[`stub'_vlabel_name]
						if `"`theLab'"'=="" {
							local theLab "`scoreVar'"
							if "`droplabels'"!="" {
								capture label drop `theLab'
								}
							}
					
						/* capture in place in case of overwriting another label */
						local theLabVals : char `ckVar'[`stub'_vlabel]
						if `"`theLabVals'"'!="" {
							capture label define `theLab' `theLabVals'
							if _rc {
								local rc = _rc
								if `rc' != 110 {
									display as error "`myname': There was a problem creating the value label " as result "`theLab'" as error "!"
									if "`keepgoing'"=="" {
										exit `rc'
										}
									local everythingOK 0
									local failreason "value label failed"
									}
								}
							}	/* end check for labelling errors */

						if `everythingOK' {
					
							/* now split off errors and scores, again */
							if "`what'"=="error" {
								/* at least need items for 0, and 1 (and -1 if need be) */
								local curlabval : label `theLab' 0
								if `"`curlabval'"' == "0" {
									label define `theLab' 0 "No errors", modify
									}
								if "`misval'"!="" {
									local curlabval : label `theLab' `misval'
									if `"`curlabval'"'=="`misval'" {
										label define `theLab' `misval' "Missing", modify
										}
									}
								local curlabval : label `theLab' 1
								if `"`curlabval'"' == "1" {
									label defin `theLab' 1 "Some error(s)", modify
									}
								}
					
							if ("`what'"=="error") | (`"`theLabVals'"'!="") {
								label values ``what'' `theLab'
								}
							}

						capture confirm new var `scoreVar'
						if !_rc {
							rename ``what'' `scoreVar'
							local now "$S_DATE at $S_TIME"
							label var `scoreVar' "`what' for `self' generated on `now'"
							if "`total'"!="" {
								quietly replace `allmark' = `allmark' + `theWt'*(`scoreVar' & (`scoreVar'<.))
								}
							}

						if "`what'"=="score" {
							local maxScore = `maxScore' + `theWt'
							}
						}	/* end check for need for the new variable */
					}	/* end of check that things are OK */

				/* this comes first to have blanket for lack of scoreVar */
				capture confirm var `scoreVar'
				if _rc {
					local scoreVar "none"
					}
				local failreason`vcnt' "`failreason'"
				/* this comes second to be sure that all info is blank in case of error */
				if "`failreason'"!="" {
					local failreasons "`failreasons' `self':`failreason';"
					local scoreVar
					local errcount
					local misscount
					}
				local errcount`vcnt' "`errcount'"
				local misscount`vcnt' "`misscount'"
				local scoreVar`vcnt' "`scoreVar'"
				local ++vcnt
				}	/* end of loop over varlist */
		
			if "`total'"!="" {
				rename `allmark' `total'
				local now "$S_DATE at $S_TIME"
				label var `total' "The total `wtmod' `what'`s' found on `now'"
				}
			return scalar maxScore = `maxScore'

			if "`what'"=="error" {
				if "`progress'"!="" {
					display _new
					}
				display as text "Results..." _newline
				
			local varwid 15
			local errmiswid 8
			local evarwid 22
			local failwid 22
				display as text "{ralign `varwid':variable name}" ///
				  " {c |} {ralign `errmiswid':Errors}" ///
				  " {c |} {ralign `errmiswid':Missing}" ///
				  " {c |} {ralign `evarwid':Error Marking Variable}" `keepgoing'
				if "`keepgoing'"!="" {
					display " {c |} {ralign `failwid':Failure Error}"
					}
				display "{hline `varwid'}{c -}{c +}{c -}{hline `errmiswid'}{c -}{c +}{c -}{hline `errmiswid'}{c -}{c +}{c -}{hline `evarwid'}" `keepgoing'
				if "`keepgoing'"!="" {
					display "{c -}{c +}{c -}{hline `failwid'}"
					}
				local vcnt 1
				foreach self of local varlist {
					local showAs "result"
					if "`scoreVar`vcnt''"=="none" {
						local showAs "text"
						}
					if "`failreason`vcnt''"!="" {
						local showAs "error"
						}
					display as `showAs' %`varwid's abbrev("`self'",`varwid') ///
					  as text " {c |} " as `showAs' "{ralign `errmiswid':`errcount`vcnt''}" ///
					  as text " {c |} " as `showAs' "{ralign `errmiswid':`misscount`vcnt''}" ///
					  as text " {c |} " as `showAs' %`evarwid's abbrev("`scoreVar`vcnt''",`evarwid') `keepgoing'
					if "`keepgoing'"!="" {
						display as text " {c |} " as error "{ralign `failwid':`failreason`vcnt''}"
						}
					local ++vcnt
					}
				}	/* end of printing for error checking */

			return local failreasons "`failreasons'"
			if "`failreasons'"!="" {
				display
				ckfail `failreasons'
				}
			}	/* end main capture block */
		local rc = _rc
		if `rc' {
			if "`preserve'"=="" {
				display _newline as text "Data restored..."
				}
			exit `rc'
			}
		else {
			if "`preserve'"=="" {
				restore, not
				}
			}
		}
end
			

program define _ckdupl, rclass
	syntax varlist [, gen(str)]

	tempvar dup
	quietly by `varlist', sort: gen byte `dup' = _n-1
	quietly count if `dup'
	local num=r(N)
	if `num' {
		disp as result  "`num' duplicate" _continue
		_pastplu, num(`num') as(result)
		disp as result " found based upon matching values of " as input "`varlist'" as result "."
		if "`gen'"!="" {
			quietly {
				by `varlist':  gen long `gen'=_n==1 & _N>1
				replace `gen'=sum(`gen') if `gen' | `dup'
				compress `gen'
				local now "$S_DATE at $S_TIME"
				label var `gen' "Duplicate group number generated on `now' based on key `varlist'"
				}
			}
		}
	else {
		disp as text "There were no duplicates based on matching values of `varlist'."
		}
	return scalar dups = `num'
end
