{marker validvalid}
{title:Valid Validation Rules}

{pstd}
There are 3 ways to create valid validation rules: give a range or set
of allowable values (the {cmd:in} syntax), give the name of another
variable which is checked in the same way (the {cmd:like} syntax), or
to use a series of Stata commands (the {cmd:complex rule}
syntax). These are explained below.
{p_end}

{marker inSyntax}
{title:The {cmd:in} syntax}

{p 8 16 2}
   {cmd:in} {c -(}{opt Stata numlist} | {opt set notation}{c )-}
{p_end}

{phang}{opt Stata numlist} is any type of list of numbers which Stata
can understand. See {help numlist} for the available syntaxes.
{p_end}

{phang}{opt set notation} allows using set notation to specify the
list. To specify a set of discrete numbers, enclose them in curly
braces ({cmd:{c -(}} and {cmd:{c )-}} and separate them with
commas. To specify ranges of numbers, use square braces {cmd:[} and
{cmd:]} to include the endpoints and and round braces {cmd:(} and
{cmd:)} to exclude endpoints. The missing value symbol {cmd:.} is used
for infinity.

{marker inExamples}
{title:Examples of {cmd:in}}

{phang}{cmd: in {c -(}3,4,5,6{c )-}}{p_end}
{pmore2}checks to see if the values are in given values. Note that
this would be identical any of the following Stata numlists{break}
{cmd:3 4 5 6}{break}
{cmd:3(1)6}{break}
{cmd:3 4 to 6}

{phang}{cmd: in [1,5]}{p_end}
{pmore2}checks to see if the values are between 1 and 5 inclusive.

{phang}{cmd: in [1,5)}{p_end}
{pmore2}checks to see if the values are from >=1 and <5.

{phang}{cmd: in [0,.]}{p_end}
{pmore2}checks to see if the values are non-negative (i.e. 0 is a
valid value). Note that the square brace (]) should really be round, but
using a round pbrace results in a Stata error.

{phang}{cmd: in [-.,0)}{p_end}
{pmore2}checks to see if the values are negative (i.e. 0 is not a valid value).

{marker likeSyntax}
{title:The {cmd:like} syntax}

{p 8 16 2}
{cmd:like} {varname}
{p_end}

{phang}{opt like} simply says that the rule given for {varname} should
be used for this variable, also. This allows having just one copy of a
rule for a series of similar variables, making it easier to change the
rule if need be.
{p_end}

{title:Example of {cmd:like}}

{phang}{cmd:like wow} will use the same validation rule as is used for
the variable {cmd:wow}.

{marker complexSyntax}
{title:The {cmd:complex} syntax}

{pstd}There really is no syntax for this, since there could be a
possible series of commands which are being run. It is best to edit
these commands by using the {help ckvaredit} dialog box and pushing
the {bf:Edit Complex Command} button. This will invoke the
{help docharedit} command, allowing the use of Stata's {help doedit}
do file editor. There are a couple of things to keep in mind:
{p_end}

{pin}The variable for which the rule is being written is refered to as
{cmd:`self'} (note the open and close quotes!). This will ensure that
the rule works properly when called using a {cmd:like} syntax or if
the variable itself is renamed.
{p_end}

{pin}If the rule is being used as a validation rule, so that valid
values will generate non-zero results and invalid values will generate
zeros, the variable holding the results is called {cmd:`valid'} (note
the quotes, again!). If the rule will flag errors, so that non-zero
results are errors, and zeros correspond to valid values, use the
variable name {cmd:`error'} (quotes required!). Finally, if the routine is a scoring
routine, then the new variable must be called {cmd:`score'} (quotes!).

{pin}Looping or branching can cause strange behavior unless the {cmd:slow}
option is used. Such things are likely not needed often for validation.{p_end}

{marker complexExamples}
{title:Examples using the {cmd:complex} syntax}

{phang}{cmd: gen byte `valid' = `self'>=0}{p_end}
{pmore2}could be used to check if the values are non-negative (i.e. >= 0).
{p_end}

{phang}{cmd: gen byte `error' = `self'<0}{p_end}
{pmore2}could also be used to check if the values are non-negative (i.e. >= 0).
{p_end}

{phang}{cmd: gen byte `valid' = `self' >= someOtherVar}{p_end}
{pmore2}could be used to check if the values of the checked variable
are at least as big as that of another variable. Note that if the
second variable is renamed, the validation rule will break.
{p_end}

{phang}{cmd: gen byte `score' = `self'=="a" + .5*(`self'=="b")}{p_end}
{pmore2}would give 1 full point (mark) for the answer "a", and 1/2
point (mark) for the answer "b".
{p_end}
